unit untTopicDownloadThread;

interface

uses
  Classes, SysUtils, Dialogs,
  untHttpClient, untStreamTool, untTool,
  untBBSCore, untTopic, untGlobal, untOption,
  untHttp3, untHttpThread, IdComponent, RegExpr,
  untBBSSub, untBoard;

type
  TTopicDownloadThread = class(THttpThread)
  protected
    FParent         : TTopic;
    FNoDownload     : Boolean;
    FTopicStateType : TTopicStateType;
    FDatFile        : TextFile;
    FNewMessage     : TTopicMessage;
    FBuffer         : TMemoryStream;
    FBufferReader   : TStreamReader;
    FReadPosition   : int64;
    FMessageCount   : integer;
    FNoFirstLine    : Boolean;
    FDatValue       : string;
    FRegex          : TRegExpr;
    FReadCgi        : boolean;
    FBufferLine     : string;
    procedure LoadLog;
    procedure HttpReceived(const Buff; Count : int64); override;
    procedure HttpStatusChange(axSender: TObject; const axStatus: TIdStatus; const asStatusText: string); override;
  public
    procedure   Execute; override;
    constructor Create(Parent: TTopic; NoDownload : Boolean);
    destructor  Destroy; override;
    procedure   RaiseMessageReceivedEvent;
  end;

implementation

{ TTopicDownloadThread }

constructor TTopicDownloadThread.Create(Parent: TTopic; NoDownload : Boolean);
var
  range : integer;
begin
  FParent := Parent;
  FParent.CountUp;

  FNoDownload := NoDownload;
  FRegex := TRegExpr.Create;

  // ͏
  case TOnlineBoard(FParent.Board).BBSType of

    bt2ch, btShitaraba:

      begin
        FRegex.Expression := '^(.*?)<>(.*?)<>(.*?)<>(.*?)<>';
        FReadCgi := false;
      end;

    btMitinoku, btJBBS:

      begin
        FRegex.Expression := '^<dt>(.+?) .+<b>(.*?)</[bB]></font>(.+?)<br><dd>(.*?)<br><br>$';

        FReadCgi := true;
      end;
  end;

  range := 0;
  if FReadCgi = false then
    if FParent.DatSize > 0 then
      range := FParent.DatSize - 1;

  inherited Create(Parent.DatUrl, range);

end;

destructor TTopicDownloadThread.Destroy;
begin
  FParent.CountDown;
  FRegex.Free;

  inherited;
end;

procedure TTopicDownloadThread.Execute;
var
  logpath        : string;
  range          : integer;
  sr             : TStreamReader;
  line           : string;
  firstchar      : char;
  msgcount       : integer;
  parse          : TTopicMessage;
  contentsize    : integer;
  blnError       : boolean;
  I              : integer;
  blnBreak       : boolean;
  starttime      : integer;
  totalsecond    : string;
  proxyitem      : TStringArray;
  proxyhost      : string;
  proxyport      : integer;
  logvalue       : string;
  msg            : TTopicMessage;
  startindex     : integer;
label
  Finish;
begin

  FParent.IsDownloadingTopic := true;
  FParent.CheckWriteFolder;

  blnError := false;

  // Oǂݍ
  LoadLog();

  if FReadCgi = false then
    if (FParent.Datsize = 0) and (FParent.MessageList.Count > 0) then
    begin
      blnError := true;
      FParent.IsError    := true;
      FParent.StatusText := '폜ځ[ŃOl܂悤ł';
      goto Finish;
    end;

  startindex := FParent.MessageList.Count;
  if FNoDownload = false then
  begin

    if Terminated = true then goto Finish;

    FBuffer := TMemoryStream.Create;
    FBufferReader := TStreamReader.Create(FBuffer);
    FReadPosition := 0;
    FMessageCount := FParent.GotMessageCount;
    FDatValue := '';
    FNoFirstLine := true;

    // ʐMJn
    inherited;

    FBuffer.Free;
    FBufferReader.Free;

    if ( 200 <= FHttp.ResponseCode) and (FHttp.ResponseCode < 300) then
    begin

      // DATTCYL^
      contentsize := FHttp.Response.ContentLength;
      if contentsize > 0 then
      if FParent.DatSize = 0 then
        FParent.DatSize := contentsize
      else
        FParent.DatSize := FParent.DatSize + contentsize - 1;

      // Last-ModifiedL^
      if blnBreak = false then
        FParent.LastModified := FHttp.Response.RawHeaders.Values['Last-Modified'];

      FParent.LastReadDate := DateTimeToStr(Now());
    end;

    FParent.NewMessageCount := FParent.MessageList.Count - FParent.GotMessageCount;
    FParent.GotMessageCount := FParent.MessageList.Count;

    if blnError = false then
    begin
      FParent.DownloadState := dsNone;

      if FParent.NewMessageCount = 0 then
        FParent.StatusText := 'VȂ'
      else
        FParent.StatusText := IntToStr(FParent.NewMessageCount) + '̃XM';
    end;

  end;

  // ÂOǂݍł̂ȂΑSo
  if FParent.OldLog = true then
  begin
    startindex := 0;
    FParent.OldLog := false;
  end;

  logvalue := '';
  for I := startindex to FParent.MessageList.Count - 1 do
  begin
    msg := TTopicMessage(FParent.MessageList[I]);
    logvalue := logvalue +
                msg.PostName  + '<>'
              + msg.Postemail + '<>'
              + msg.RestStr   + '<>'
              + msg.Body + #10;
  end;

  // DATt@C̏o
  logpath := FParent.LocalDir + FParent.TopicId + '.d';
  AssignFile(FDatFile, logpath);
  if FileExists(logpath) = false then ReWrite(FDatFile);
  Append(FDatFile);
  Write(FDatFile, logvalue);
  CloseFile(FDatFile);

Finish:

  FParent.SaveIdx();

  FParent.DownloadState := dsNone;
  FParent.IsDownloadingTopic := false;

  //Synchronize(FreeThread);
  Terminate;
  //Free;

end;

procedure TTopicDownloadThread.HttpReceived(const Buff; Count: int64);
var
  line  : string;
  parse : TTopicMessage;
  ignoreNomatch : boolean;
  msgIndex   : integer;
  msgName    : string;
  msgEmail   : string;
  msgRestStr : string;
  msgBody    : string;
  msg        : TTopicMessage;
  match      : boolean;
begin

  FBuffer.Seek(0, soFromEnd);
  FBuffer.Write(Buff, Count);
  FBuffer.Seek(FReadPosition, soFromBeginning);

  while FBufferReader.ReadLine(line) do
  begin

    // Ol܂`FbN
    if FReadCgi = false then
      if FNoFirstLine then
      begin
        FNoFirstLine := false;
        if FParent.DatSize > 0 then
          if line = '' then
          begin
            continue;
          end else
          begin
            FParent.IsError    := true;
            FParent.StatusText := '폜ځ[ŃOl܂悤ł';
            exit;
          end;
      end;

    FReadPosition := FBuffer.Position;
    //Inc(FMessageCount);

    if FRegex.Exec(line) then
    begin

      if FReadCgi = false then
      begin
        Inc(FMessageCount);
        msgIndex   := FMessageCount;
        msgName    := FRegex.Substitute('$1');
        msgEmail   := FRegex.Substitute('$2');
        msgRestStr := FRegex.Substitute('$3');
        msgBody    := FRegex.Substitute('$4');
      end else
      begin
        Inc(FMessageCount);
        msgIndex   := StrToIntNeo(FRegex.Substitute('$1'));
        msgName    := FRegex.Substitute('$2');
        msgEmail   := '';
        msgRestStr := FRegex.Substitute('$3');
        msgBody    := FRegex.Substitute('$4');
      end;

      // bZ[WNX
      if msgIndex = FMessageCount then
      begin
        msg := TTopicMessage.Create();
        msg.Index        := msgIndex;
        msg.PostName     := msgName;
        msg.PostEmail    := msgEmail;
        msg.RestStr      := msgRestStr;
        msg.Body         := msgBody;
        msg.IsNewMessage := true;
        FParent.AddNewMessage(msg);
      end;

    end else
      if FReadCgi = false then
        ShowMessage('̓G[');

  end;

  Synchronize(RaiseMessageReceivedEvent);

  FBuffer.Seek(0, soFromEnd);

end;

procedure TTopicDownloadThread.HttpStatusChange(axSender: TObject;
  const axStatus: TIdStatus; const asStatusText: string);
begin

  case axStatus of

    hsConnecting :

    begin

      FParent.StatusText := FHttp.Host + 'ɐڑ';

    end;

    hsConnected :

    begin

      FParent.DownloadState := dsProcessing;

      FParent.StatusText := FHttp.Host + 'ɐڑ܂';;

    end;

  end;

end;

{ --------------------------------------------------------
  ֐: LoadLog
  pr  : O̓ǂݍ
    : Ȃ
  ߂l: Ȃ
  l  : Ȃ
  ------------------------------------------------------ }
procedure TTopicDownloadThread.LoadLog;
var
  localloaded : boolean;
  I, J        : Integer;
  msg         : TTopicMessage;
  msgcount    : Integer;
  logpath     : string;
  logvalue    : string;
  lines       : TStringList;
  parse       : TTopicMessage;
  items       : TStringArray;
  msgName     : string;
  msgEmail    : string;
  msgRestStr  : string;
  msgBody     : string;
begin

  msgcount := FParent.MessageList.Count;

  if FParent.IsLoadedLocalDat = false then
  begin

    lines := TStringList.Create;

    logpath := FParent.LocalDir + FParent.TopicId + '.d';
    if FileExists(logpath) = true then
    begin

   	  // [Jǂݍ
      lines.LoadFromFile(logpath);
      for I := 0 to lines.Count - 1 do
      begin
        items := Split(lines[I], '<>');
        msgName    := items[0];
        msgEmail   := items[1];
        msgRestStr := items[2];
        msgBody    := items[3];

        msg := TTopicMessage.Create;
        msg.PostName  := msgName;
        msg.PostEmail := msgEmail;
        msg.RestStr   := msgRestStr;
        msg.Body      := msgBody;
        FParent.AddNewMessage(msg);

        localloaded := true;

      end;

    end else
    begin

      // Of[^AÂ``
      // ÔȂ炻炩ǂݍ
      logpath := FParent.LocalDir + FParent.TopicId + '.dat';
      if FileExists(logpath) = true then
      begin

        lines.LoadFromFile(logpath);
        for I := 0 to lines.Count - 1 do
        begin
          items := Split(lines[I], ',');
          for J := 0 to 3 do
            items[J] := StringReplace(items[J], 'M', ',', [rfReplaceAll]);

          msgName    := items[0];
          msgEmail   := items[1];
          msgRestStr := items[2];
          msgBody    := items[3];

          msg := TTopicMessage.Create;
          msg.PostName  := msgName;
          msg.PostEmail := msgEmail;
          msg.RestStr   := msgRestStr;
          msg.Body      := msgBody;
          FParent.AddNewMessage(msg);

          localloaded := true;

        end;
      end;
    end;
    FParent.IsLoadedLocalDat := true;

    lines.Free;    
  end;
	FParent.Gotmessagecount := FParent.MessageList.Count;

  //if localloaded = true then
  //begin
    Synchronize(RaiseMessageReceivedEvent);

    FParent.DownloadState := dsDatLoaded;
  //end;
  
end;

procedure TTopicDownloadThread.RaiseMessageReceivedEvent;
begin
  if Assigned(FParent.OnMessageReceived) then
    FParent.OnMessageReceived(FParent);
end;

end.
